from pylab import show, figure
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.figure import Figure

def make_single_plot():
    fig = Figure()
    fig = figure(1)
    ax = fig.add_subplot(111)
    ax.grid(True)
    return ax

def interactive_display(ax):
    ax.set_figure(figure(1))
    show()

def save_to_file(ax, filename):
    canvas = FigureCanvas(ax.get_figure())
    canvas.print_figure(filename)

def func1(x1, x2):
    sx1 =  x1 * 4 - 1
    sx2 =  x2 * 2 - 1
    return sqrt(sx1 ** 2 + sx2 ** 2)

def func3(x1, x2):
    sx1 =  x1 * 4 - 1
    sx2 =  x2 * 2 - 1
    return (1 - sx1 / 2 + sx1 ** 5 + sx2 ** 3) * exp(- sx1 ** 2 - sx2 ** 2)

if __name__ == '__main__':
    from pylab import *
    from gumby.model import SvmModel

    N = 7 # sqrt of the number of points in the training set
    G = 100 # resolution of the display grid
    GRID_SEARCH = False
    f = func3

    x1 = x2 = arange(0., 1., 1. / N)

    x1, x2 = meshgrid(x1, x2)
    train_x1, train_x2 = x1.flatten(), x2.flatten()
    train_x = array([train_x1, train_x2]).transpose()

    train_y = f(train_x1, train_x2)

    # build and train a SVM on this data

    m = SvmModel()
    m.gamma = 10
    m.cost = 1000
    m.kernel_type = 'rbf'
    m.model_type = 'epsilon'
    if True:
        m.find_parameters(train_x, train_y, nr_fold=2,
                          do_model_type=True, do_kernel_type=False)
    m.train(train_x, train_y)

    # build a mesh to test the SVM predictions
    x1 = x2 = arange(0., 1., 1. / G)
    X1, X2 = meshgrid(x1, x2)
    Y = f(X1, X2)

    flat_mesh = array([X1.flatten(), X2.flatten()]).transpose()
    predicted = m.predict_forward(flat_mesh)
    predicted = predicted.reshape((len(x1), len(x2)))

    # display the predicted values on a color map
    ax = make_single_plot()
    colormap = ax.pcolor(X1, X2, predicted, shading='flat')
    ax.get_figure().colorbar(colormap)

    # display the training data
    ax.scatter(train_x1, train_x2, marker="o", c='white')

    # show the contour lines of the theoritical solution
    cs = ax.contour(X1, X2, Y, colors='k')
    cs.clabel(fontsize=9, inline=1)

    # open the display window
    interactive_display(ax)

